"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FSHelpers = void 0;
const util_1 = __importDefault(require("util"));
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
class FSHelpers {
    static async exists(path) {
        let fx = false;
        try {
            await util_1.default.promisify(fs_1.default.stat)(path);
            fx = true;
        }
        catch (ex) {
            /* noop */
        }
        return fx;
    }
    static async rmDir(folder) {
        if (!(await FSHelpers.exists(folder)))
            return;
        const readdir = util_1.default.promisify(fs_1.default.readdir);
        let entries = await readdir(folder, { withFileTypes: true });
        await Promise.all(entries.map(async (entry) => {
            // skip .nfs files in the EFS storage layer
            if (entry.name.startsWith('.nfs'))
                return;
            let fullPath = path_1.default.join(folder, entry.name);
            return entry.isDirectory() ? FSHelpers.rmDir(fullPath) : FSHelpers.safeUnlinkFile(fullPath);
        }));
        await util_1.default.promisify(fs_1.default.rmdir)(folder);
    }
    /**
     * Unlinks a file, but does not throw if unlinking fails.
     * Supports passing an undefined value to reduce caller boilerplate
     * when called with optional arguments.
     * @param path
     */
    static async safeUnlinkFile(path) {
        try {
            await util_1.default.promisify(fs_1.default.unlink)(path);
        }
        catch (ex) {
            /* noop */
        }
    }
    /**
     * Recursively parse all sub-folders in a given folder, calling the given callback for any files
     * @param rootPath Folder to parse
     * @param callback Callback to call for each file
     */
    static readAllFiles(rootPath, callback) {
        const parseRecursively = (path) => {
            for (const filename of fs_1.default.readdirSync(path)) {
                // Ignore any files beginning with . e.g. .DS_Store, as well as any readme files
                if (filename.startsWith('.') || filename.toLowerCase().includes('readme')) {
                    continue;
                }
                const filePath = path_1.default.join(path, filename);
                if (fs_1.default.statSync(filePath).isDirectory()) {
                    // Recursively parse any directories
                    parseRecursively(filePath);
                }
                else {
                    // Process a file
                    callback({
                        filename,
                        path,
                    });
                }
            }
        };
        parseRecursively(rootPath);
    }
}
exports.FSHelpers = FSHelpers;
//# sourceMappingURL=fs-helpers.js.map