"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.upload = exports.VALID_EXTENSIONS = exports.EXTENSION_MAPPING = void 0;
const form_data_1 = __importDefault(require("form-data"));
const node_fetch_1 = __importDefault(require("node-fetch"));
const http_1 = __importDefault(require("http"));
const https_1 = __importDefault(require("https"));
const encoding_1 = __importDefault(require("../shared/encoding"));
const path_1 = __importDefault(require("path"));
const keepAliveAgentHttp = new http_1.default.Agent({ keepAlive: true });
const keepAliveAgentHttps = new https_1.default.Agent({ keepAlive: true });
exports.EXTENSION_MAPPING = {
    '.wav': 'audio/wav',
    '.cbor': 'application/cbor',
    '.json': 'application/json',
    '.jpg': 'image/jpeg',
    '.jpeg': 'image/jpeg',
    '.png': 'image/png',
    '.csv': 'text/csv',
    '.txt': 'text/plain',
    '.mp4': 'video/mp4',
    '.avi': 'video/avi',
};
exports.VALID_EXTENSIONS = Object.keys(exports.EXTENSION_MAPPING);
async function upload(opts) {
    if (opts.buffer.length > 100 * 1024 * 1024) {
        throw new Error('File too large, max. size is 100MB');
    }
    let ext = path_1.default.extname(opts.filename).toLowerCase();
    let headers = {
        'x-api-key': opts.apiKey,
        'x-upload-source': 'EDGE_IMPULSE_CLI_UPLOADER',
        'Connection': 'keep-alive'
    };
    if (opts.label.type === 'label') {
        headers['x-label'] = encoding_1.default(opts.label.label);
    }
    else if (opts.label.type === 'unlabeled') {
        headers['x-label'] = encoding_1.default('');
        headers['x-no-label'] = '1';
    }
    if (!opts.allowDuplicates) {
        headers['x-disallow-duplicates'] = '1';
    }
    if (opts.boundingBoxes) {
        headers['x-bounding-boxes'] = JSON.stringify(opts.boundingBoxes);
    }
    if (opts.metadata) {
        headers['x-metadata'] = JSON.stringify(opts.metadata);
    }
    if (opts.addDateId) {
        headers['x-add-date-id'] = '1';
    }
    headers['x-device-type'] = 'EDGE_IMPULSE_CLI';
    let agent = opts.config.endpoints.internal.ingestion.indexOf('https:') === 0 ?
        keepAliveAgentHttps :
        keepAliveAgentHttp;
    let category = opts.category;
    const form = new form_data_1.default();
    form.append('data', opts.buffer, {
        filename: opts.filename,
        contentType: exports.EXTENSION_MAPPING[ext] || 'application/octet-stream',
    });
    let res = await node_fetch_1.default(opts.config.endpoints.internal.ingestion + '/api/' + category + '/files', {
        method: 'POST',
        headers: headers,
        body: form,
        agent: agent,
        compress: true,
    });
    let body = await res.text();
    let msg;
    try {
        msg = JSON.parse(body);
    }
    catch (ex2) {
        // noop
    }
    if (res.status !== 200) {
        if (msg && msg.success === false) {
            throw new Error(msg.error);
        }
        throw new Error(body || res.status.toString());
    }
    if (!msg) {
        throw new Error('Ingestion returned 200, but body is not a valid response message: ' +
            body);
    }
    if (!msg.success) {
        throw new Error(msg.error);
    }
    for (let f of msg.files) {
        if (!f.success) {
            throw new Error(f.error);
        }
    }
}
exports.upload = upload;
//# sourceMappingURL=make-image.js.map