"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EdgeImpulseApi = void 0;
// tslint:disable-next-line: variable-name, no-var-requires
const PATH = require('path');
// tslint:disable-next-line: no-unsafe-any
module.paths.push(PATH.join(process.cwd(), 'node_modules'));
const api_1 = require("./sdk/api");
const ws_1 = __importDefault(require("ws"));
const JOB_CONNECTION_TIMEOUT = 60000;
const DEBUG_PREFIX = '\x1b[33m[API]\x1b[0m';
class EdgeImpulseApi {
    /**
     * Initialize the API
     * @param opts An object with { auth, endpoint }.
     */
    constructor(opts) {
        let endpoint = (opts || {}).endpoint || 'https://studio.edgeimpulse.com';
        if (endpoint.endsWith('/')) {
            endpoint = endpoint.slice(0, endpoint.length - 1);
        }
        if (endpoint.endsWith('/v1')) {
            endpoint = endpoint.slice(0, endpoint.length - 3);
        }
        this._opts = {
            endpoint,
            debug: (opts === null || opts === void 0 ? void 0 : opts.debug) || false,
            extraHeaders: (opts === null || opts === void 0 ? void 0 : opts.extraHeaders) || {},
        };
        this.admin = new api_1.AdminApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.auth = new api_1.AuthApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.classify = new api_1.ClassifyApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.dsp = new api_1.DSPApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.deployment = new api_1.DeploymentApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.devices = new api_1.DevicesApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.export = new api_1.ExportApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.health = new api_1.HealthApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.impulse = new api_1.ImpulseApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.jobs = new api_1.JobsApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.learn = new api_1.LearnApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.login = new api_1.LoginApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.metrics = new api_1.MetricsApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.optimization = new api_1.OptimizationApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.organizationBlocks = new api_1.OrganizationBlocksApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.organizationCreateProject = new api_1.OrganizationCreateProjectApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.organizationData = new api_1.OrganizationDataApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.organizationDataCampaigns = new api_1.OrganizationDataCampaignsApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.organizationJobs = new api_1.OrganizationJobsApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.organizationPipelines = new api_1.OrganizationPipelinesApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.organizationPortals = new api_1.OrganizationPortalsApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.organizations = new api_1.OrganizationsApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.performanceCalibration = new api_1.PerformanceCalibrationApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.projects = new api_1.ProjectsApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.rawData = new api_1.RawDataApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.themes = new api_1.ThemesApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.thirdPartyAuth = new api_1.ThirdPartyAuthApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.uploadPortal = new api_1.UploadPortalApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.user = new api_1.UserApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
        this.whitelabels = new api_1.WhitelabelsApi(this._opts.endpoint + '/v1', { extraHeaders: this._opts.extraHeaders });
    }
    async authenticate(opts) {
        if (!opts || !(opts instanceof Object)) {
            throw new Error('Missing auth object as first argument to `authenticate`');
        }
        if (opts.method === 'apiKey') {
            if (!opts.apiKey || typeof opts.apiKey !== 'string') {
                throw new Error('Missing "auth.apiKey"');
            }
            if (!opts.apiKey.startsWith('ei_')) {
                throw new Error('"auth.apiKey" should start with "ei_"');
            }
        }
        else if (opts.method === 'jwtToken') {
            if (!opts.jwtToken || typeof opts.jwtToken !== 'string') {
                throw new Error('Missing "auth.jwtToken"');
            }
        }
        else {
            throw new Error('Invalid value for "auth.method", should be apiKey or jwtToken');
        }
        const jwtTokenAuthId = 2; // JWT Token header
        const apiKeyAuthId = 0; // API Key header
        if (opts.method === 'apiKey') {
            this.admin.setApiKey(apiKeyAuthId, opts.apiKey);
            this.auth.setApiKey(apiKeyAuthId, opts.apiKey);
            this.classify.setApiKey(apiKeyAuthId, opts.apiKey);
            this.dsp.setApiKey(apiKeyAuthId, opts.apiKey);
            this.deployment.setApiKey(apiKeyAuthId, opts.apiKey);
            this.devices.setApiKey(apiKeyAuthId, opts.apiKey);
            this.export.setApiKey(apiKeyAuthId, opts.apiKey);
            this.impulse.setApiKey(apiKeyAuthId, opts.apiKey);
            this.jobs.setApiKey(apiKeyAuthId, opts.apiKey);
            this.learn.setApiKey(apiKeyAuthId, opts.apiKey);
            this.optimization.setApiKey(apiKeyAuthId, opts.apiKey);
            this.organizationBlocks.setApiKey(apiKeyAuthId, opts.apiKey);
            this.organizationCreateProject.setApiKey(apiKeyAuthId, opts.apiKey);
            this.organizationData.setApiKey(apiKeyAuthId, opts.apiKey);
            this.organizationDataCampaigns.setApiKey(apiKeyAuthId, opts.apiKey);
            this.organizationJobs.setApiKey(apiKeyAuthId, opts.apiKey);
            this.organizationPipelines.setApiKey(apiKeyAuthId, opts.apiKey);
            this.organizationPortals.setApiKey(apiKeyAuthId, opts.apiKey);
            this.organizations.setApiKey(apiKeyAuthId, opts.apiKey);
            this.performanceCalibration.setApiKey(apiKeyAuthId, opts.apiKey);
            this.projects.setApiKey(apiKeyAuthId, opts.apiKey);
            this.rawData.setApiKey(apiKeyAuthId, opts.apiKey);
            this.themes.setApiKey(apiKeyAuthId, opts.apiKey);
            this.thirdPartyAuth.setApiKey(apiKeyAuthId, opts.apiKey);
            this.uploadPortal.setApiKey(apiKeyAuthId, opts.apiKey);
            this.user.setApiKey(apiKeyAuthId, opts.apiKey);
            this.whitelabels.setApiKey(apiKeyAuthId, opts.apiKey);
            this.admin.setApiKey(jwtTokenAuthId, undefined);
            this.auth.setApiKey(jwtTokenAuthId, undefined);
            this.classify.setApiKey(jwtTokenAuthId, undefined);
            this.dsp.setApiKey(jwtTokenAuthId, undefined);
            this.deployment.setApiKey(jwtTokenAuthId, undefined);
            this.devices.setApiKey(jwtTokenAuthId, undefined);
            this.export.setApiKey(jwtTokenAuthId, undefined);
            this.impulse.setApiKey(jwtTokenAuthId, undefined);
            this.jobs.setApiKey(jwtTokenAuthId, undefined);
            this.learn.setApiKey(jwtTokenAuthId, undefined);
            this.optimization.setApiKey(jwtTokenAuthId, undefined);
            this.organizationBlocks.setApiKey(jwtTokenAuthId, undefined);
            this.organizationCreateProject.setApiKey(jwtTokenAuthId, undefined);
            this.organizationData.setApiKey(jwtTokenAuthId, undefined);
            this.organizationDataCampaigns.setApiKey(jwtTokenAuthId, undefined);
            this.organizationJobs.setApiKey(jwtTokenAuthId, undefined);
            this.organizationPipelines.setApiKey(jwtTokenAuthId, undefined);
            this.organizationPortals.setApiKey(jwtTokenAuthId, undefined);
            this.organizations.setApiKey(jwtTokenAuthId, undefined);
            this.performanceCalibration.setApiKey(jwtTokenAuthId, undefined);
            this.projects.setApiKey(jwtTokenAuthId, undefined);
            this.rawData.setApiKey(jwtTokenAuthId, undefined);
            this.themes.setApiKey(jwtTokenAuthId, undefined);
            this.thirdPartyAuth.setApiKey(jwtTokenAuthId, undefined);
            this.uploadPortal.setApiKey(jwtTokenAuthId, undefined);
            this.user.setApiKey(jwtTokenAuthId, undefined);
            this.whitelabels.setApiKey(jwtTokenAuthId, undefined);
        }
        else {
            let jwtToken = opts.jwtToken;
            this.admin.setApiKey(jwtTokenAuthId, jwtToken);
            this.auth.setApiKey(jwtTokenAuthId, jwtToken);
            this.classify.setApiKey(jwtTokenAuthId, jwtToken);
            this.dsp.setApiKey(jwtTokenAuthId, jwtToken);
            this.deployment.setApiKey(jwtTokenAuthId, jwtToken);
            this.devices.setApiKey(jwtTokenAuthId, jwtToken);
            this.export.setApiKey(jwtTokenAuthId, jwtToken);
            this.impulse.setApiKey(jwtTokenAuthId, jwtToken);
            this.jobs.setApiKey(jwtTokenAuthId, jwtToken);
            this.learn.setApiKey(jwtTokenAuthId, jwtToken);
            this.optimization.setApiKey(jwtTokenAuthId, jwtToken);
            this.organizationBlocks.setApiKey(jwtTokenAuthId, jwtToken);
            this.organizationCreateProject.setApiKey(jwtTokenAuthId, jwtToken);
            this.organizationData.setApiKey(jwtTokenAuthId, jwtToken);
            this.organizationDataCampaigns.setApiKey(jwtTokenAuthId, jwtToken);
            this.organizationJobs.setApiKey(jwtTokenAuthId, jwtToken);
            this.organizationPipelines.setApiKey(jwtTokenAuthId, jwtToken);
            this.organizationPortals.setApiKey(jwtTokenAuthId, jwtToken);
            this.organizations.setApiKey(jwtTokenAuthId, jwtToken);
            this.performanceCalibration.setApiKey(jwtTokenAuthId, jwtToken);
            this.projects.setApiKey(jwtTokenAuthId, jwtToken);
            this.rawData.setApiKey(jwtTokenAuthId, jwtToken);
            this.themes.setApiKey(jwtTokenAuthId, jwtToken);
            this.thirdPartyAuth.setApiKey(jwtTokenAuthId, jwtToken);
            this.uploadPortal.setApiKey(jwtTokenAuthId, jwtToken);
            this.user.setApiKey(jwtTokenAuthId, jwtToken);
            this.whitelabels.setApiKey(jwtTokenAuthId, jwtToken);
            this.admin.setApiKey(apiKeyAuthId, undefined);
            this.auth.setApiKey(apiKeyAuthId, undefined);
            this.classify.setApiKey(apiKeyAuthId, undefined);
            this.dsp.setApiKey(apiKeyAuthId, undefined);
            this.deployment.setApiKey(apiKeyAuthId, undefined);
            this.devices.setApiKey(apiKeyAuthId, undefined);
            this.export.setApiKey(apiKeyAuthId, undefined);
            this.impulse.setApiKey(apiKeyAuthId, undefined);
            this.jobs.setApiKey(apiKeyAuthId, undefined);
            this.learn.setApiKey(apiKeyAuthId, undefined);
            this.optimization.setApiKey(apiKeyAuthId, undefined);
            this.organizationBlocks.setApiKey(apiKeyAuthId, undefined);
            this.organizationCreateProject.setApiKey(apiKeyAuthId, undefined);
            this.organizationData.setApiKey(apiKeyAuthId, undefined);
            this.organizationDataCampaigns.setApiKey(apiKeyAuthId, undefined);
            this.organizationJobs.setApiKey(apiKeyAuthId, undefined);
            this.organizationPipelines.setApiKey(apiKeyAuthId, undefined);
            this.organizationPortals.setApiKey(apiKeyAuthId, undefined);
            this.organizations.setApiKey(apiKeyAuthId, undefined);
            this.performanceCalibration.setApiKey(apiKeyAuthId, undefined);
            this.projects.setApiKey(apiKeyAuthId, undefined);
            this.rawData.setApiKey(apiKeyAuthId, undefined);
            this.themes.setApiKey(apiKeyAuthId, undefined);
            this.thirdPartyAuth.setApiKey(apiKeyAuthId, undefined);
            this.uploadPortal.setApiKey(apiKeyAuthId, undefined);
            this.user.setApiKey(apiKeyAuthId, undefined);
            this.whitelabels.setApiKey(apiKeyAuthId, undefined);
        }
    }
    async runJobUntilCompletion(opts, dataCallback) {
        if (typeof opts !== 'object') {
            throw new Error('Missing options object');
        }
        if (opts.type === 'project') {
            return this.runProjectJobUntilCompletion(opts, dataCallback);
        }
        else if (opts.type === 'organization') {
            return this.runOrgJobUntilCompletion(opts, dataCallback);
        }
        else {
            throw new Error('options.type should be either "project" or "organization"');
        }
    }
    async runProjectJobUntilCompletion(opts, dataCallback) {
        if (typeof opts !== 'object') {
            throw new Error('Missing options object');
        }
        if (typeof opts.projectId !== 'number') {
            throw new Error('Missing "projectId"');
        }
        if (typeof opts.jobId !== 'number') {
            throw new Error('Missing "jobId"');
        }
        const { projectId, jobId } = opts;
        return this.waitForJobImpl(jobId, dataCallback, () => this.getProjectWebsocket(projectId), async () => {
            const d = await this.jobs.getJobStatus(projectId, jobId);
            if (this._opts.debug) {
                console.log(DEBUG_PREFIX, 'runJobUntilCompletion', 'projectId', projectId, 'jobId', jobId, 'status', d);
            }
            return d;
        });
    }
    async getProjectWebsocket(projectId) {
        const tokenRes = await this.projects.getSocketToken(projectId);
        return this.getWebsocketImpl(tokenRes);
    }
    async runOrgJobUntilCompletion(opts, dataCallback) {
        if (typeof opts !== 'object') {
            throw new Error('Missing options object');
        }
        if (typeof opts.organizationId !== 'number') {
            throw new Error('Missing "organizationId"');
        }
        if (typeof opts.jobId !== 'number') {
            throw new Error('Missing "jobId"');
        }
        const { organizationId, jobId } = opts;
        return this.waitForJobImpl(jobId, dataCallback, () => this.getOrgWebsocket(organizationId), async () => {
            const d = await this.organizationJobs.getOrganizationJobStatus(organizationId, jobId);
            if (this._opts.debug) {
                console.log(DEBUG_PREFIX, 'runJobUntilCompletion', 'organizationId', organizationId, 'jobId', jobId, 'status', d);
            }
            return d;
        });
    }
    async getOrgWebsocket(organizationId) {
        const tokenRes = await this.organizationJobs.getOrganizationSocketToken(organizationId);
        return this.getWebsocketImpl(tokenRes);
    }
    async sleep(ms) {
        return new Promise((resolve) => setTimeout(resolve, ms));
    }
    async waitForJobImpl(jobId, dataCallback, getWebsocketFn, getJobStatusFn) {
        let terminated = false;
        const connectToSocket = async () => {
            let socket;
            // Get a websocket (max 60 sec timeout)
            let getWebsocketTimeout = Date.now() + JOB_CONNECTION_TIMEOUT;
            while (1) {
                try {
                    socket = await getWebsocketFn();
                    if (dataCallback) {
                        dataCallback('Connected to job\n');
                    }
                    break;
                }
                catch (ex2) {
                    let ex = ex2;
                    if (Date.now() > getWebsocketTimeout) {
                        throw ex;
                    }
                    if (this._opts.debug) {
                        console.log(DEBUG_PREFIX, 'Failed to get socket token, retrying in 5 seconds...');
                    }
                    if (dataCallback) {
                        dataCallback(`Failed to connect to job (${ex.message || ex.toString()}), retrying in 5 seconds...\n`);
                    }
                    await this.sleep(5000);
                }
            }
            let pingIv = setInterval(() => {
                if (socket && socket.readyState === ws_1.default.OPEN) {
                    socket.ping();
                    socket.send('2');
                }
            }, 5000);
            socket.onmessage = (msg) => {
                try {
                    let m = JSON.parse(msg.data.toString().replace(/^[0-9]+/, ''));
                    // tslint:disable-next-line: no-unsafe-any
                    if (m[0] !== `job-data-${jobId}` && m[0] !== `job-finished-${jobId}`)
                        return;
                    // tslint:disable-next-line: no-unsafe-any
                    if (m[1].data) {
                        if (dataCallback) {
                            // tslint:disable-next-line: no-unsafe-any
                            dataCallback(m[1].data);
                        }
                    }
                }
                catch (e) {
                    /* noop */
                }
            };
            socket.onclose = async () => {
                clearInterval(pingIv);
                if (terminated)
                    return;
                if (dataCallback) {
                    dataCallback('Lost connection to job, retrying to connect...\n');
                }
                try {
                    await connectToSocket();
                }
                catch (ex2) {
                    let ex = ex2;
                    if (dataCallback) {
                        dataCallback(`Failed to open new socket (${ex.message || ex.toString()}\n`);
                    }
                }
            };
            return socket;
        };
        let s = await connectToSocket();
        let lastJobStatusReqSucceeded = Date.now();
        while (1) {
            await this.sleep(5000);
            let jobHasFailed = false;
            try {
                const d = await getJobStatusFn();
                lastJobStatusReqSucceeded = Date.now();
                let job = d.job || { finishedSuccessful: undefined };
                if (job.finishedSuccessful === true) {
                    break;
                }
                else if (job.finishedSuccessful === false) {
                    jobHasFailed = true;
                }
            }
            catch (ex2) {
                let ex = ex2;
                if (this._opts.debug) {
                    console.log(DEBUG_PREFIX, 'Failed to check job status', ex.message || ex.toString());
                }
                if (Date.now() - lastJobStatusReqSucceeded > JOB_CONNECTION_TIMEOUT) {
                    throw new Error('Failed to check job status for 60 seconds: ' +
                        (ex.message || ex.toString()));
                }
            }
            if (jobHasFailed) {
                throw new Error('Job failed');
            }
        }
        terminated = true;
        s.terminate();
    }
    async getWebsocketImpl(tokenRes) {
        const wsHost = this._opts.endpoint.replace('http', 'ws');
        let tokenData = {
            success: true,
            token: tokenRes.token || { socketToken: '' },
        };
        let ws = new ws_1.default(wsHost + '/socket.io/?token=' +
            tokenData.token.socketToken + '&EIO=3&transport=websocket');
        return new Promise((resolve, reject) => {
            ws.onclose = () => {
                reject('websocket was closed');
            };
            ws.onerror = err => {
                reject('websocket error: ' + err);
            };
            ws.onmessage = msg => {
                try {
                    let m = JSON.parse(msg.data.toString().replace(/^[0-9]+/, ''));
                    if (m[0] === 'hello') {
                        // tslint:disable-next-line: no-unsafe-any
                        if (m[1].hello && m[1].hello.version === 1) {
                            clearTimeout(rejectTimeout);
                            // console.log('Connected to job websocket');
                            resolve(ws);
                        }
                        else {
                            reject(JSON.stringify(m[1]));
                        }
                    }
                }
                catch (ex) {
                    /* noop */
                }
            };
            let rejectTimeout = setTimeout(() => {
                reject('Did not authenticate with the websocket API within 10 seconds');
            }, 10000);
        });
    }
}
exports.EdgeImpulseApi = EdgeImpulseApi;
//# sourceMappingURL=api.js.map